#include <VSslClient>
#include <VDebugNew>

REGISTER_METACLASS(VSslClient, VNet)

// ----------------------------------------------------------------------------
// VSslClient
// ----------------------------------------------------------------------------
VSslClient::VSslClient(void* owner) : VNetClient(owner)
{
  sslSession = new VSslSession(this);
  host       = "";
  port       = 0;
  localHost  = "";
  localPort  = 0;
}

VSslClient::~VSslClient()
{
  close();
  SAFE_DELETE(sslSession);
}

bool VSslClient::doOpen()
{
  if (port == 0)
  {
    SET_ERROR(VSslError, "port is zero", VERR_PORT_IS_ZERO);
    return false;
  }

  sslSession->socket.setPeerVerifyMode(QSslSocket::VerifyNone);
  sslSession->socket.connectToHostEncrypted(host, (quint16)port);
  bool res = sslSession->socket.waitForEncrypted(10000); // gilgil temp 2013.12.07
  if (!res)
  {
    QString err = sslSession->socket.errorString();
    SET_DEBUG_ERROR(VError, qPrintable(err), (int)sslSession->socket.error());
    return false;
  }
  if (!sslSession->open())
  {
    error = sslSession->error;
    return false;
  }

  return true;
}

bool VSslClient::doClose()
{
  sslSession->close();
  return true;
}

int  VSslClient::doRead(char* buf, int size)
{
  int res = sslSession->read(buf, size);
  if (res == VERR_FAIL)
    error = sslSession->error;
  return res;
}

int  VSslClient::doWrite(char* buf, int size)
{
  int res = sslSession->write(buf, size);
  if (res == VERR_FAIL)
    error = sslSession->error;
  return res;
}

void VSslClient::load(VXml xml)
{
  VNetClient::load(xml);
  host = xml.getStr("host", host);
  port = xml.getInt("port", port);
  localHost = xml.getStr("localHost", localHost);
  localPort = xml.getInt("localPort", localPort);
}

void VSslClient::save(VXml xml)
{
  VNetClient::save(xml);
  xml.setStr("host", host);
  xml.setInt("port", port);
  if (localHost != "") xml.setStr("localHost", localHost);
  if (localPort != 0)  xml.setInt("localPort", localPort);
}

#ifdef QT_GUI_LIB
void VSslClient::addOptionWidget(QLayout* layout)
{
  VNetClient::addOptionWidget(layout);
  edHost      = VShowOption::addLineEdit(layout, "Host", host);
  edPort      = VShowOption::addLineEdit(layout, "Port", QString::number(port));
  edLocalHost = localHost != "" ? VShowOption::addLineEdit(layout, "Local Host", localHost) : NULL;
  edLocalPort = localPort != 0 ? VShowOption::addLineEdit(layout, "Local Port", QString::number(localPort)) : NULL;
}

void VSslClient::saveOption(QDialog* parent)
{
  VNetClient::saveOption(parent);
  host      = edHost->text();
  port      = edPort->text().toInt();
  if (edLocalHost != NULL) localHost = edLocalHost->text();
  if (edLocalPort != NULL) localPort = edLocalPort->text().toInt();
}
#endif // QT_GUI_LIB
